@@ -1,15 +1,16 @@
 
-from typing import Union, Tuple, Sequence, Iterable, FrozenSet
+from collections.abc import Iterable, Sequence
+from typing import Union
 
 __all__ = ['whitespace', 'find_any', 'get_raw']
 
 whitespace = frozenset(b' \t\n\r\x0b\f')
 
-_Line = Tuple[int, int, int]
+_Line = tuple[int, int, int]
 _Lines = Sequence[_Line]
 
 
-def find_any(data: Union[bytes, memoryview], end_marker: FrozenSet[int],
+def find_any(data: Union[bytes, memoryview], end_marker: frozenset[int],
              start: int, end: int, inverse: bool, reverse: bool) -> int:
     if reverse:
         range_iter: Iterable[int] = reversed(range(start, end))
@@ -28,7 +29,10 @@ def find_any(data: Union[bytes, memoryview], end_marker: FrozenSet[int],
 def get_raw(view: memoryview, *lines: _Lines) -> memoryview:
     try:
         start = lines[0][0][0]
-        end = lines[-1][-1][2]
-        return view[start:end]
     except IndexError:
-        return memoryview(b'')
+        start = 0
+    try:
+        end = lines[-1][-1][2]
+    except IndexError:
+        end = -1
+    return view[start:end]
