@@ -1,20 +1,24 @@
 
+from __future__ import annotations
+
 import re
-from typing import Tuple, Sequence, List, Iterable, Optional, ClassVar
+from collections.abc import Iterable, Sequence
+from typing import Optional, ClassVar
 
 from . import CommandSelect, CommandNoArgs
 from .. import Params, Space, EndLine
 from ..exceptions import NotParseable
-from ..primitives import Atom, ListP
+from ..primitives import Atom, List
 from ..specials import AString, Mailbox, SequenceSet, Flag, FetchAttribute, \
     SearchKey, ExtensionOptions
 from ...bytes import rev
 from ...flags import FlagOp
 
 __all__ = ['CheckCommand', 'CloseCommand', 'ExpungeCommand', 'CopyCommand',
-           'FetchCommand', 'StoreCommand', 'SearchCommand', 'UidCommand',
-           'UidCopyCommand', 'UidExpungeCommand', 'UidFetchCommand',
-           'UidSearchCommand', 'UidStoreCommand', 'IdleCommand']
+           'MoveCommand', 'FetchCommand', 'StoreCommand', 'SearchCommand',
+           'UidCommand', 'UidCopyCommand', 'UidMoveCommand',
+           'UidExpungeCommand', 'UidFetchCommand', 'UidSearchCommand',
+           'UidStoreCommand', 'IdleCommand']
 
 
 class CheckCommand(CommandNoArgs, CommandSelect):
@@ -63,7 +67,7 @@ class ExpungeCommand(CommandSelect):
 
     @classmethod
     def parse(cls, buf: memoryview, params: Params) \
-            -> Tuple['ExpungeCommand', memoryview]:
+            -> tuple[ExpungeCommand, memoryview]:
         uid_set: Optional[SequenceSet] = None
         if params.uid:
             _, buf = Space.parse(buf, params)
@@ -101,7 +105,45 @@ class CopyCommand(CommandSelect):
 
     @classmethod
     def parse(cls, buf: memoryview, params: Params) \
-            -> Tuple['CopyCommand', memoryview]:
+            -> tuple[CopyCommand, memoryview]:
+        _, buf = Space.parse(buf, params)
+        seq_set, buf = SequenceSet.parse(buf, params)
+        _, buf = Space.parse(buf, params)
+        mailbox, buf = Mailbox.parse(buf, params)
+        _, buf = EndLine.parse(buf, params)
+        return cls(params.tag, seq_set, mailbox), buf
+
+
+class MoveCommand(CommandSelect):
+    """The ``MOVE`` command moves messages from the selected mailbox to the
+    end of the destination mailbox.
+
+    See Also:
+        `RFC 6851 <https://tools.ietf.org/html/rfc6851>`_
+
+    Args:
+        tag: The command tag.
+        seq_set: The sequence set of the messages to move.
+        mailbox: The destination mailbox.
+
+    """
+
+    command = b'MOVE'
+    uid: ClassVar[bool] = False
+
+    def __init__(self, tag: bytes, seq_set: SequenceSet,
+                 mailbox: Mailbox) -> None:
+        super().__init__(tag)
+        self.sequence_set = seq_set
+        self.mailbox_obj = mailbox
+
+    @property
+    def mailbox(self) -> str:
+        return str(self.mailbox_obj)
+
+    @classmethod
+    def parse(cls, buf: memoryview, params: Params) \
+            -> tuple[MoveCommand, memoryview]:
         _, buf = Space.parse(buf, params)
         seq_set, buf = SequenceSet.parse(buf, params)
         _, buf = Space.parse(buf, params)
@@ -133,13 +175,12 @@ class FetchCommand(CommandSelect):
                  options: ExtensionOptions = None) -> None:
         super().__init__(tag)
         self.sequence_set = seq_set
-        self.no_expunge_response = not seq_set.uid
         self.attributes = attr_list
         self.options = options or ExtensionOptions.empty()
 
     @classmethod
     def _check_macros(cls, buf: memoryview, params: Params) \
-            -> Tuple[Sequence[FetchAttribute], memoryview]:
+            -> tuple[Sequence[FetchAttribute], memoryview]:
         atom, after = Atom.parse(buf, params)
         macro = atom.value.upper()
         if macro == b'ALL':
@@ -164,7 +205,7 @@ class FetchCommand(CommandSelect):
 
     @classmethod
     def parse(cls, buf: memoryview, params: Params) \
-            -> Tuple['FetchCommand', memoryview]:
+            -> tuple[FetchCommand, memoryview]:
         _, buf = Space.parse(buf, params)
         seq_set, buf = SequenceSet.parse(buf, params)
         _, buf = Space.parse(buf, params)
@@ -180,7 +221,7 @@ class FetchCommand(CommandSelect):
             pass
         if not attr_list:
             params_copy = params.copy(list_expected=[FetchAttribute])
-            attr_list_p, buf = ListP.parse(buf, params_copy)
+            attr_list_p, buf = List.parse(buf, params_copy)
             attr_list = attr_list_p.get_as(FetchAttribute)
         if params.uid:
             attr_list = list(attr_list) + [FetchAttribute(b'UID')]
@@ -222,7 +263,7 @@ class StoreCommand(CommandSelect):
 
     @classmethod
     def _parse_store_info(cls, buf: memoryview, params: Params) \
-            -> Tuple[FlagOp, bool, memoryview]:
+            -> tuple[FlagOp, bool, memoryview]:
         info, after = Atom.parse(buf, params)
         match = cls._info_pattern.match(info.value)
         if not match:
@@ -233,15 +274,15 @@ class StoreCommand(CommandSelect):
 
     @classmethod
     def _parse_flag_list(cls, buf: memoryview, params: Params) \
-            -> Tuple[Sequence[Flag], memoryview]:
+            -> tuple[Sequence[Flag], memoryview]:
         try:
             params_copy = params.copy(list_expected=[Flag])
-            flag_list_p, buf = ListP.parse(buf, params_copy)
+            flag_list_p, buf = List.parse(buf, params_copy)
         except NotParseable:
             pass
         else:
             return flag_list_p.get_as(Flag), buf
-        flag_list: List[Flag] = []
+        flag_list: list[Flag] = []
         while True:
             try:
                 flag, buf = Flag.parse(buf, params)
@@ -252,7 +293,7 @@ class StoreCommand(CommandSelect):
 
     @classmethod
     def parse(cls, buf: memoryview, params: Params) \
-            -> Tuple['StoreCommand', memoryview]:
+            -> tuple[StoreCommand, memoryview]:
         _, buf = Space.parse(buf, params)
         seq_set, buf = SequenceSet.parse(buf, params)
         options, buf = ExtensionOptions.parse(buf, params)
@@ -291,7 +332,7 @@ class SearchCommand(CommandSelect):
 
     @classmethod
     def _parse_charset(cls, buf: memoryview, params: Params) \
-            -> Tuple[Optional[str], memoryview]:
+            -> tuple[Optional[str], memoryview]:
         try:
             _, after = Space.parse(buf, params)
             atom, after = Atom.parse(after, params)
@@ -311,7 +352,7 @@ class SearchCommand(CommandSelect):
 
     @classmethod
     def _parse_options(cls, buf: memoryview, params: Params) \
-            -> Tuple[ExtensionOptions, memoryview]:
+            -> tuple[ExtensionOptions, memoryview]:
         start = cls._whitespace_length(buf)
         if buf[start:start + 6] == b'RETURN':
             return ExtensionOptions.parse(buf[start + 6:], params)
@@ -321,7 +362,7 @@ class SearchCommand(CommandSelect):
 
     @classmethod
     def parse(cls, buf: memoryview, params: Params) \
-            -> Tuple['SearchCommand', memoryview]:
+            -> tuple[SearchCommand, memoryview]:
         options, buf = cls._parse_options(buf, params)
         charset, buf = cls._parse_charset(buf, params)
         search_keys = []
@@ -355,8 +396,8 @@ class UidCommand(CommandSelect):
 
     @classmethod
     def parse(cls, buf: memoryview, params: Params) \
-            -> Tuple['UidCommand', memoryview]:
-        raise NotImplementedError
+            -> tuple[UidCommand, memoryview]:
+        raise TypeError(cls)  # never parsed directly
 
 
 class UidCopyCommand(CopyCommand):
@@ -371,13 +412,32 @@ class UidCopyCommand(CopyCommand):
 
     @classmethod
     def parse(cls, buf: memoryview, params: Params) \
-            -> Tuple['UidCopyCommand', memoryview]:
+            -> tuple[UidCopyCommand, memoryview]:
         ret, buf = super().parse(buf, params.copy(uid=True))
         if not isinstance(ret, UidCopyCommand):
             raise TypeError(ret)
         return ret, buf
 
 
+class UidMoveCommand(MoveCommand):
+    """The ``UID MOVE`` variant of the ``MOVE`` command, which uses message
+    UIDs instead of sequence numbers.
+
+    """
+
+    command = b'UID MOVE'
+    delegate = MoveCommand
+    uid = True
+
+    @classmethod
+    def parse(cls, buf: memoryview, params: Params) \
+            -> tuple[UidMoveCommand, memoryview]:
+        ret, buf = super().parse(buf, params.copy(uid=True))
+        if not isinstance(ret, UidMoveCommand):
+            raise TypeError(ret)
+        return ret, buf
+
+
 class UidExpungeCommand(ExpungeCommand):
     """The ``UID EXPUNGE`` variant of the ``EXPUNGE`` command, which uses
     message UIDs instead of sequence numbers.
@@ -390,7 +450,7 @@ class UidExpungeCommand(ExpungeCommand):
 
     @classmethod
     def parse(cls, buf: memoryview, params: Params) \
-            -> Tuple['UidExpungeCommand', memoryview]:
+            -> tuple[UidExpungeCommand, memoryview]:
         ret, buf = super().parse(buf, params.copy(uid=True))
         if not isinstance(ret, UidExpungeCommand):
             raise TypeError(ret)
@@ -409,7 +469,7 @@ class UidFetchCommand(FetchCommand):
 
     @classmethod
     def parse(cls, buf: memoryview, params: Params) \
-            -> Tuple['UidFetchCommand', memoryview]:
+            -> tuple[UidFetchCommand, memoryview]:
         ret, buf = super().parse(buf, params.copy(uid=True))
         if not isinstance(ret, UidFetchCommand):
             raise TypeError(ret)
@@ -428,7 +488,7 @@ class UidSearchCommand(SearchCommand):
 
     @classmethod
     def parse(cls, buf: memoryview, params: Params) \
-            -> Tuple['UidSearchCommand', memoryview]:
+            -> tuple[UidSearchCommand, memoryview]:
         ret, buf = super().parse(buf, params.copy(uid=True))
         if not isinstance(ret, UidSearchCommand):
             raise TypeError(ret)
@@ -447,7 +507,7 @@ class UidStoreCommand(StoreCommand):
 
     @classmethod
     def parse(cls, buf: memoryview, params: Params) \
-            -> Tuple['UidStoreCommand', memoryview]:
+            -> tuple[UidStoreCommand, memoryview]:
         ret, buf = super().parse(buf, params.copy(uid=True))
         if not isinstance(ret, UidStoreCommand):
             raise TypeError(ret)
@@ -478,11 +538,11 @@ class IdleCommand(CommandSelect):
 
     @classmethod
     def parse(cls, buf: memoryview, params: Params) \
-            -> Tuple['IdleCommand', memoryview]:
+            -> tuple[IdleCommand, memoryview]:
         _, buf = EndLine.parse(buf, params)
         return cls(params.tag), buf
 
-    def parse_done(self, buf: memoryview) -> Tuple[bool, memoryview]:
+    def parse_done(self, buf: memoryview) -> tuple[bool, memoryview]:
         """Parse the continuation line sent by the client to end the ``IDLE``
         command.
 
