@@ -1,4 +1,5 @@
-from typing import Tuple
+
+from __future__ import annotations
 
 from .. import Params, Parseable
 from ..exceptions import NotParseable
@@ -29,7 +30,7 @@ class Tag(Parseable[bytes]):
 
     @classmethod
     def parse(cls, buf: memoryview, params: Params) \
-            -> Tuple['Tag', memoryview]:
+            -> tuple[Tag, memoryview]:
         start = cls._whitespace_length(buf)
         match = cls._pattern.match(buf, start)
         if not match:
